<?php
/**
 * mPDF PHP library.
 * https://mpdf.github.io
 */
require dirname( __DIR__, 1 ) . '/vendor/autoload.php';

use Mpdf\Mpdf;


defined( 'ABSPATH' ) || exit;


/**
 * Export PDF class.
 */
class Billy_PDF_Export {

	/**
	 * Stylesheet.
	 *
	 * @access public
	 * @var string
	 */
	public static $pdfstyles;

	/**
	 * Font directory.
	 *
	 * @access public
	 * @var string
	 */
	public static $pdffont_dir;

	/**
	 * Font data.
	 *
	 * @access public
	 * @var array
	 */
	public static $pdffont;

	/**
	 * On load.
	 */
	public function __construct() {
		self::$pdfstyles   = file_get_contents( dirname( __DIR__ ) . '/mpdf/css/pdf.css' );
		self::$pdffont_dir = array(
			dirname( __DIR__ ) . '/mpdf/fonts/',
		);
		self::$pdffont     = array(
			'R' => 'Roboto-Regular.ttf',
			'B' => 'Roboto-Bold.ttf',
		);

		if ( current_user_can( 'read_private_posts' ) ) {
			$this->init();
		}
	}

	/**
	 * Plugin initiation.
	 *
	 * A helper function to initiate actions, hooks and other features needed.
	 */
	public function init() {
		add_action( 'rest_api_init', array( $this, 'billy_rest_api_init' ) );
	}

	/**
	 * WP-API initiation.
	 *
	 * GET /wp-json/export/pdf/{ID}
	 */
	public function billy_rest_api_init() {
		register_rest_route(
			'export',
			'/pdf',
			array(
				'methods'  => 'GET',
				'callback' => array( $this, 'billy_export_pdf' ),
			)
		);
	}

	/**
	 * WP-API Custom endpoint callbacks.
	 *
	 * @param array $data Options for the function.
	 * @return json object of post data.
	 */
	public function billy_export_pdf( $data ) {
		$parameters = $data->get_params();
		$post_id    = esc_attr( $parameters['id'] );
		$post_type  = get_post_type( $post_id );
		$reference  = esc_attr( get_the_title( $post_id ) );

		$css = static::$pdfstyles;
		/*if ( ! empty( $parameters['stylesheets'] ) ) {
			$enqueued_styles = base64_decode( esc_attr( $parameters['stylesheets'] ) );
			$enqueued_styles = explode( ',', str_replace( array( '[', ']', '"' ), '', $enqueued_styles ) );

			// Add all registered stylesheets to output.
			foreach ( $enqueued_styles as $enqueued_style ) {
				$css .= file_get_contents( $enqueued_style );
			}
		}*/

		// Debugging.
		//print_r( $css );exit();
		//print_r( str_replace( '"', '', base64_decode( $parameters['stylesheets'] ) ) );exit();
		//print_r( $post_type );exit();
		//print_r( apply_filters( 'the_content', get_post_field( 'post_content', $post_id ) ) );exit();

		// Create PDF.
		$mpdf = new Mpdf(
			array(
				'tempDir'      => dirname( __DIR__, 1 ) . '/mpdf/tmp',
				'simpleTables' => true,
				'mode'         => 'utf-8',
				'fontDir'      => static::$pdffont_dir,
				'fontdata'     => array(
					'pdffont' => static::$pdffont,
				),
				'default_font' => 'pdffont',
			)
		);

		$mpdf->WriteHTML( $css, \Mpdf\HTMLParserMode::HEADER_CSS );

		$content = '';
		if ( ! empty( $post_id ) && get_post( $post_id ) ) {
			global $post;
			$post = get_post( $post_id );

			setup_postdata( $post );

			$blocks = parse_blocks( get_the_content() );
			foreach ( $blocks as $block ) {
				$content .= render_block( $block );
			}

			wp_reset_postdata();

			$mpdf->SetHTMLFooter(
				'<table width="100%">
					<tr>
						<td width="33%"><small>' . esc_attr( date_i18n( get_option( 'date_format' ), date( 'U' ) ) ) . '</small></td>
						<td width="33%" align="center"><small>{PAGENO}/{nbpg}</small></td>
						<td width="33%" align="right"><small>' . $reference . '</small></td>
					</tr>
				</table>'
			);
		}

		$output = '<html>
			<body class="' . $post_type . '-template-default single single-' . $post_type . ' singular"><div class="entry-content"><div id="' . $post_type . '" class="' . $post_type . '-wrapper">' . $content . '</div></div>
			</body>
		</html>';

		$mpdf->SetTitle( $reference );
		$mpdf->WriteHTML( $output, \Mpdf\HTMLParserMode::HTML_BODY );
		$mpdf->Output( $reference . '.pdf', 'I' );
		exit();
	}

}
